"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BuerliGeometry = void 0;
const react_1 = __importDefault(require("react"));
const suspend_react_1 = require("suspend-react");
const hooks_1 = require("../hooks");
const BuerliGeometryInteraction_1 = require("./BuerliGeometryInteraction");
const Product_1 = require("./Product");
let didWarn = false;
exports.BuerliGeometry = react_1.default.memo(props => {
    var _a;
    const { selection = false, drawingId, suspend: shouldSuspend = false } = props;
    const activeDrawingId = (0, hooks_1.useBuerli)(state => state.drawing.active);
    const actualDrawingId = drawingId !== null && drawingId !== void 0 ? drawingId : activeDrawingId;
    if (Array.isArray(shouldSuspend) && didWarn === false) {
        console.warn('Array notation for <BuerliGeometry suspend={[".Load", ".SetDatabaseSettings", ...]}> is deprecated. Use <BuerliGeometry suspend=".Load,.SetDatabaseSettings" /> instead.');
        didWarn = true;
    }
    // Get pending states
    const pending = (0, hooks_1.useDrawing)(actualDrawingId, state => state.cad.pending);
    const filterItems = typeof shouldSuspend === 'string' ? shouldSuspend.split(',') : Array.isArray(shouldSuspend) ? shouldSuspend : [];
    const filter = typeof shouldSuspend === 'function'
        ? shouldSuspend
        : (data) => typeof shouldSuspend === 'boolean' ? shouldSuspend : filterItems.find(entry => data.name.includes(entry));
    // Collect all pending promises if shouldSuspend is true
    const hashes = (_a = pending === null || pending === void 0 ? void 0 : pending.filter(data => data.promise && filter(data))) !== null && _a !== void 0 ? _a : [];
    const promises = hashes.map(data => data.promise);
    // Suspend all pending promises. This prevents the rendering of the component until all promises are resolved.
    // It will also allow external components to interface with the results. A useEffect in the same <Suspense>
    // boundary will guarantee the presence of all loaded products and geometry.
    if (shouldSuspend)
        (0, suspend_react_1.suspend)(() => Promise.all(promises), ['buerli/react/BuerliGeometry', ...promises]);
    if (actualDrawingId === null)
        return null;
    return selection ? (react_1.default.createElement(BuerliGeometryInteraction_1.BuerliGeometryInteraction, { drawingId: actualDrawingId },
        react_1.default.createElement(Product_1.Product, Object.assign({}, props, { drawingId: actualDrawingId, isRoot: true })))) : (react_1.default.createElement(Product_1.Product, Object.assign({}, props, { drawingId: actualDrawingId, isRoot: true })));
});
