"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GlobalTransform = void 0;
const core_1 = require("@buerli.io/core");
const react_1 = __importDefault(require("react"));
const THREE = __importStar(require("three"));
const hooks_1 = require("../hooks");
/**
 * Goes up the object tree starting from the given objectId
 * and accumulates the transformations of all predecessors resulting in the global matrix of the given objectId.
 * These transformations are reactive, i.e. every time some subobject's matrix changes, the final transformation is recalculated as well.
 * When root is reached it wraps children into a group and applies resulting matrix to it.
 *
 * @param drawingId - id of drawing
 * @param objectId - id of object whoes parents should be taken in account
 * @param matrix - needed for recursive calls
 */
const GlobalTransform = ({ drawingId, objectId, matrix = new THREE.Matrix4(), children }) => {
    // For some reason, this component will be rendered even if the corresponding object does
    // not exist in the tree anymore. So its existance needs to be checked!
    const parentId = (0, hooks_1.useDrawing)(drawingId, drawing => { var _a; return (_a = drawing.structure.tree[objectId]) === null || _a === void 0 ? void 0 : _a.parent; });
    const csys = (0, hooks_1.useDrawing)(drawingId, drawing => { var _a; return (_a = drawing.structure.tree[objectId]) === null || _a === void 0 ? void 0 : _a.coordinateSystem; });
    const matrix_ = react_1.default.useMemo(() => {
        if (!csys) {
            return matrix.clone();
        }
        const local = core_1.MathUtils.convertToMatrix4(csys);
        return local.multiply(matrix);
    }, [csys, matrix]);
    return parentId ? (react_1.default.createElement(exports.GlobalTransform, { drawingId: drawingId, objectId: parentId, matrix: matrix_ }, children)) : (react_1.default.createElement("group", { matrix: matrix_, matrixAutoUpdate: false }, children));
};
exports.GlobalTransform = GlobalTransform;
