"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PointCloud = void 0;
const react_1 = __importDefault(require("react"));
const fiber_1 = require("@react-three/fiber");
const THREE = __importStar(require("three"));
const pointimage_png_1 = __importDefault(require("./pointimage.png"));
const defaultColor = 'black';
const PointCloud = props => {
    const { geometry, productId, containerId, config } = props;
    const pointConfig = config === null || config === void 0 ? void 0 : config.points;
    const { invalidate } = (0, fiber_1.useThree)();
    const posAttr = react_1.default.useRef(null);
    const colorAttr = react_1.default.useRef(null);
    const isActive = (config === null || config === void 0 ? void 0 : config.disabled) === undefined ? true : !config.disabled;
    const [positions, colors, lookup] = react_1.default.useMemo(() => {
        const color = new THREE.Color((pointConfig === null || pointConfig === void 0 ? void 0 : pointConfig.color) || defaultColor);
        const positions1 = [];
        const colors1 = [];
        const lookupMap = {};
        if (geometry) {
            for (let i = 0; i < geometry.points.length; i++) {
                const pt = geometry.points[i];
                positions1.push(pt.position.x);
                positions1.push(pt.position.y);
                positions1.push(pt.position.z);
                colors1.push(color.r);
                colors1.push(color.g);
                colors1.push(color.b);
                colors1.push((pointConfig === null || pointConfig === void 0 ? void 0 : pointConfig.hidden) ? 0.0 : (pointConfig === null || pointConfig === void 0 ? void 0 : pointConfig.opacity) || 1.0); // alpha
                lookupMap[`${productId}|${pt.graphicId}|${pt.type}`] = i;
            }
        }
        return [new Float32Array(positions1), new Float32Array(colors1), lookupMap];
    }, [geometry, pointConfig, productId]);
    const [uniforms] = react_1.default.useState(() => ({
        size: { value: 10 },
        tex: { value: new THREE.TextureLoader().load(pointimage_png_1.default) },
    }));
    react_1.default.useEffect(() => {
        if (posAttr.current) {
            posAttr.current.needsUpdate = true;
        }
        if (colorAttr.current) {
            colorAttr.current.needsUpdate = true;
        }
        invalidate();
    }, [positions, colors, lookup, posAttr, colorAttr, invalidate]);
    return (react_1.default.createElement("points", { renderOrder: 400, frustumCulled: false, userData: { pointMap: geometry.points, productId, containerId, isBuerliGeometry: isActive } },
        react_1.default.createElement("bufferGeometry", null,
            react_1.default.createElement("bufferAttribute", { ref: posAttr, attach: 'attributes-position', count: positions.length / 3, array: positions, itemSize: 3 }),
            react_1.default.createElement("bufferAttribute", { ref: colorAttr, attach: 'attributes-rgba', count: colors.length / 4, array: colors, itemSize: 4 })),
        react_1.default.createElement("shaderMaterial", { attach: "material", uniforms: uniforms, vertexShader: vert, fragmentShader: frag, alphaTest: 0.5, depthWrite: false, depthTest: true, vertexColors: true, transparent: true })));
};
exports.PointCloud = PointCloud;
const vert = `uniform float size;
attribute vec4 rgba;
varying vec4 vColor;
void main() {
  gl_Position = projectionMatrix * modelViewMatrix * vec4(position, 1.0);
  gl_PointSize = size;
  vColor = rgba;
}`;
const frag = `uniform sampler2D tex;
varying vec4 vColor;
void main() {
  gl_FragColor = vColor;
	gl_FragColor = gl_FragColor * texture2D(tex, gl_PointCoord);
}`;
