"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Point = void 0;
const fiber_1 = require("@react-three/fiber");
const react_1 = __importDefault(require("react"));
const CameraHelper_1 = require("../../../helpers/CameraHelper");
const isVector3 = (entity) => entity && entity.isVector3;
const Point = props => {
    const { elem, color = 'black', renderOrder = 1, opacity = 1, pointSize = 6.0, onClick } = props;
    const { camera, size } = (0, fiber_1.useThree)();
    const pointMeshRef = react_1.default.useRef(null);
    const calcScale = react_1.default.useCallback((canvasRect) => {
        const point = elem;
        const sf = CameraHelper_1.CameraHelper.calculateScaleFactor(point.position, pointSize, camera, canvasRect);
        return [sf, sf, sf];
    }, [camera, elem, pointSize]);
    const updateScale = react_1.default.useCallback((canvasRect) => {
        const sf = calcScale(canvasRect);
        pointMeshRef.current && pointMeshRef.current.scale.set(sf[0], sf[1], sf[2]);
    }, [calcScale]);
    (0, fiber_1.useFrame)(renderProps => updateScale(renderProps.size));
    react_1.default.useEffect(() => updateScale(size), [size, updateScale]);
    const pointGeom = elem;
    return isVector3(pointGeom.position) ? (react_1.default.createElement("mesh", { onClick: e => onClick === null || onClick === void 0 ? void 0 : onClick([elem], e), ref: pointMeshRef, scale: calcScale(size), position: pointGeom.position, userData: elem, renderOrder: renderOrder },
        react_1.default.createElement("sphereGeometry", { args: [1.2, 32, 16] }),
        react_1.default.createElement("meshBasicMaterial", { color: color, transparent: true, toneMapped: false }))) : null;
};
exports.Point = Point;
