"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginManager = void 0;
const core_1 = require("@buerli.io/core");
const react_1 = __importDefault(require("react"));
const shallow_1 = __importDefault(require("zustand/shallow"));
const hooks_1 = require("../../hooks");
const PluginInstance = ({ drawingId, objId }) => {
    const activeFeature = (0, hooks_1.useDrawing)(drawingId, d => d.plugin.active.feature);
    const isVisible = (0, hooks_1.useDrawing)(drawingId, d => d.plugin.visible.includes(objId));
    const unsub = react_1.default.useRef();
    // Create plugin if it's activated and hasn't been created yet
    react_1.default.useEffect(() => {
        const drawing = (0, core_1.getDrawing)(drawingId);
        if (drawing) {
            // Find object in the structure-tree
            const obj = drawing.structure.tree[objId];
            const isCreated = drawing.plugin.refs[objId] !== undefined;
            if (obj && (activeFeature === objId || isVisible) && !isCreated) {
                // Look for a plugin that fits the description
                const plugins = core_1.api.getState().options.plugins;
                const impl = plugins && plugins[obj.class];
                if (impl) {
                    // If there's one we create a plugin and return the unsubscribe function, so that it will remove itself on unmount
                    const def = typeof impl === 'function' ? impl() : impl;
                    unsub.current = drawing.api.plugin.createPlugin(def, obj.id, drawingId, obj.id);
                }
            }
        }
    }, [drawingId, objId, activeFeature, isVisible]);
    // Remove plugin on PluginInstace component unoumt (regularly it happens when corresponding object is removed from tree)
    react_1.default.useEffect(() => {
        return () => {
            const unsub_ = unsub.current;
            const drawing = (0, core_1.getDrawing)(drawingId);
            if (unsub_ && drawing) {
                unsub_(objId === drawing.plugin.active.feature);
            }
        };
    }, [drawingId, objId]);
    // This component has no view
    return null;
};
const PluginDrawing = ({ drawingId }) => {
    const ids = (0, hooks_1.useDrawing)(drawingId, state => Object.getOwnPropertyNames(state.structure.tree), shallow_1.default);
    return (react_1.default.createElement(react_1.default.Fragment, null, ids === null || ids === void 0 ? void 0 : ids.map(id => (react_1.default.createElement(PluginInstance, { key: id, drawingId: drawingId, objId: Number(id) })))));
};
const PluginManagerImpl = () => {
    const drawingIds = (0, hooks_1.useBuerli)(state => state.drawing.ids);
    return (react_1.default.createElement(react_1.default.Fragment, null, drawingIds.map(drawingId => (react_1.default.createElement(PluginDrawing, { key: drawingId, drawingId: drawingId })))));
};
exports.PluginManager = react_1.default.memo(PluginManagerImpl);
