"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginWrapperView = void 0;
const core_1 = require("@buerli.io/core");
const react_1 = __importDefault(require("react"));
const hooks_1 = require("../../hooks");
const Plugin_1 = __importDefault(require("./../Plugin"));
const useAncestors = (drawingId, objectId) => {
    return react_1.default.useMemo(() => {
        const drawing = (0, core_1.getDrawing)(drawingId);
        if (!objectId || !drawing) {
            return [];
        }
        const tree = drawing.structure.tree;
        let curId = objectId;
        const ancestors = [];
        while (Boolean(curId) && tree[curId]) {
            ancestors.push(tree[curId].parent);
            curId = tree[curId].parent;
        }
        return ancestors;
    }, [drawingId, objectId]);
};
const PluginWrapperView = props => {
    const { drawingId, pluginId, showOnlyActive = true } = props;
    const objectId = (0, hooks_1.usePlugin)(drawingId, pluginId, plugin => plugin.objectId);
    const ancestors = useAncestors(drawingId, objectId);
    const currentProduct = (0, hooks_1.useDrawing)(drawingId, d => d.structure.currentProduct) || -1;
    const isActive = ancestors.indexOf(currentProduct) !== -1;
    const visible = (0, hooks_1.useDrawing)(drawingId, d => d.plugin.visible) || [];
    const globPlugins = (0, hooks_1.useDrawing)(drawingId, d => d.plugin.active.global) || [];
    const featurePlugin = (0, hooks_1.useDrawing)(drawingId, d => d.plugin.active.feature);
    const useIt = showOnlyActive ? isActive : true;
    const showIt = (useIt && visible.indexOf(pluginId) >= 0) || globPlugins.indexOf(pluginId) >= 0 || pluginId === featurePlugin;
    return showIt ? react_1.default.createElement(Plugin_1.default, { drawingId: drawingId, pluginId: pluginId, view: true }) : null;
};
exports.PluginWrapperView = PluginWrapperView;
