"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Messenger = void 0;
const icons_1 = require("@ant-design/icons");
const notification_1 = __importDefault(require("antd/lib/notification"));
const react_1 = __importDefault(require("react"));
const hooks_1 = require("../../hooks");
const defaultStyle = {
    margin: 5,
    padding: 3,
    borderRadius: 5,
};
const defaultConfig = {
    placement: 'bottomRight',
};
const showNotification = (msg, id, style) => {
    const args = {
        key: id,
        message: msg.title ? msg.title : msg.type,
        description: msg.text.length > 120 ? msg.text.substring(0, 120) + ' ...' : msg.text,
        style,
    };
    switch (msg.type) {
        case 'error':
            args.duration = 0;
            notification_1.default.error(args);
            break;
        case 'info':
            notification_1.default.info(args);
            break;
        case 'busy':
            args.icon = react_1.default.createElement(icons_1.LoadingOutlined, { style: { fontSize: 24 }, spin: true });
            args.duration = 0;
            notification_1.default.info(args);
            break;
        case 'success':
            notification_1.default.success(args);
            break;
        case 'warning':
            notification_1.default.warning(args);
            break;
        default:
            notification_1.default.open(args);
            break;
    }
};
const Messenger = ({ config = defaultConfig, style = defaultStyle, }) => {
    const ts = (0, hooks_1.useBuerli)(state => state.message.timestamp);
    const msg = (0, hooks_1.useBuerli)(state => state.message.data);
    react_1.default.useEffect(() => {
        notification_1.default.config(config);
    }, [config]);
    react_1.default.useMemo(() => {
        msg && (msg.text || msg.title) && showNotification(msg, msg.id, style);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [ts, msg, style]);
    return null;
};
exports.Messenger = Messenger;
exports.default = exports.Messenger;
