"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findBuerliGeometry = void 0;
const isPoint = (intersection) => { var _a, _b; return Boolean((_b = (_a = intersection.object) === null || _a === void 0 ? void 0 : _a.userData) === null || _b === void 0 ? void 0 : _b.pointMap); };
const isLine = (intersection) => { var _a, _b; return Boolean((_b = (_a = intersection.object) === null || _a === void 0 ? void 0 : _a.userData) === null || _b === void 0 ? void 0 : _b.lineMap); };
const isMesh = (intersection) => { var _a, _b; return Boolean((_b = (_a = intersection.object) === null || _a === void 0 ? void 0 : _a.userData) === null || _b === void 0 ? void 0 : _b.meshMap); };
const getProductElement = (intersection) => {
    let result = null;
    const userData = intersection.object.userData;
    const index = intersection.index;
    const faceIndex = intersection.faceIndex;
    if (typeof index === 'number' && isPoint(intersection)) {
        // Point
        const point = userData.pointMap[index];
        if (point) {
            result = Object.assign(Object.assign({}, point), { productId: userData.productId, intersection });
        }
    }
    else if (typeof index === 'number' && isLine(intersection)) {
        // Line
        const line = userData.lineMap[index || 0];
        if (line) {
            result = Object.assign(Object.assign({}, line), { productId: userData.productId, intersection });
        }
    }
    else if (typeof faceIndex === 'number' && isMesh(intersection)) {
        // Mesh
        const mesh = userData.meshMap[faceIndex || 0];
        if (mesh) {
            result = Object.assign(Object.assign({}, mesh), { productId: userData.productId, intersection });
        }
    }
    return result;
};
const findBuerliGeometry = (e, lineThreshold, pointThreshold) => {
    const buerliItems = e.intersections.filter(i => { var _a; return (_a = i.object.userData) === null || _a === void 0 ? void 0 : _a.isBuerliGeometry; });
    if (buerliItems.length === 0) {
        return [];
    }
    const getThreshold = (intersection) => (isPoint(intersection) ? pointThreshold : lineThreshold);
    const minDist = buerliItems[0].distance;
    const within = buerliItems.filter(i => Math.abs(minDist - i.distance) <= getThreshold(i));
    const outside = buerliItems.filter(i => Math.abs(minDist - i.distance) > getThreshold(i));
    const withinSorted = within.sort((a, b) => b.object.renderOrder - a.object.renderOrder);
    return withinSorted
        .concat(outside)
        .map(i => getProductElement(i))
        .filter(i => Boolean(i));
};
exports.findBuerliGeometry = findBuerliGeometry;
