"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useEmbeddedPlugin = void 0;
const core_1 = require("@buerli.io/core");
const react_1 = __importStar(require("react"));
const Plugin_1 = require("../components/Plugin");
/**
 * UseEmbeddedPlugin might be called twice for the same plugin,
 * but embedded plugin should be created and remove once.
 * Creating plugin once - can be implemented by checking of plugin existing.
 * For removing plugin once we hanve to count usages of this plugin,
 * and when counter equals to 0 remove plugin from store.
 *
 * Increasing and decreasing usageCounter happens in last UseEffect.
 **/
const usageCounters = {};
const unsubs = {};
function useEmbeddedPlugin(definition, drawingId, parentPluginId, objectId, onCreated) {
    const [attributes] = (0, react_1.useState)(() => ({
        drawingId,
        objectId,
        pluginId: parentPluginId + '/embed/' + definition.description.name,
    }));
    // Fetch plugin from store
    const plugin = (0, core_1.getDrawing)(drawingId).plugin.refs[attributes.pluginId];
    // If it exists already, we are ready to roll ...
    const [ready, setReady] = (0, react_1.useState)(!!plugin);
    // If not we have to create it
    (0, react_1.useEffect)(() => {
        const plugin_ = (0, core_1.getDrawing)(drawingId).plugin.refs[attributes.pluginId];
        if (!plugin_) {
            // Create plugin
            const pluginApi = (0, core_1.getDrawing)(drawingId).api.plugin;
            const unsub = pluginApi.createPlugin(definition, attributes.pluginId, drawingId, objectId);
            usageCounters[attributes.pluginId] = 0;
            // Flag ready state
            unsubs[attributes.pluginId] = unsub;
        }
        // Here plugin exists and now we have to render it.
        setReady(true);
    }, [definition, drawingId, objectId, plugin, attributes.pluginId]);
    (0, react_1.useEffect)(() => {
        // Increase referece count on mount.
        usageCounters[attributes.pluginId]++;
        return () => {
            // Decrease referece count on mount and remove plugin if refCount === 0.
            if (--usageCounters[attributes.pluginId] === 0) {
                unsubs[attributes.pluginId](false);
                delete unsubs[attributes.pluginId];
            }
        };
    }, [drawingId, attributes.pluginId]);
    const onCreatedRef = react_1.default.useRef(onCreated);
    (0, react_1.useEffect)(() => {
        onCreatedRef.current && onCreatedRef.current(attributes.pluginId);
    }, [attributes.pluginId]);
    // Return plugins root and view
    return [
        (props) => (ready && definition.Root ? react_1.default.createElement(definition.Root, Object.assign({}, props, attributes)) : null),
        (props) => (ready && definition.View ? react_1.default.createElement(Plugin_1.View, Object.assign({ additionalProps: props }, attributes)) : null),
    ];
}
exports.useEmbeddedPlugin = useEmbeddedPlugin;
