import React from 'react';
import { suspend } from 'suspend-react';
import { useBuerli, useDrawing } from '../hooks';
import { BuerliGeometryInteraction } from './BuerliGeometryInteraction';
import { Product } from './Product';
let didWarn = false;
export const BuerliGeometry = React.memo(props => {
    var _a;
    const { selection = false, drawingId, suspend: shouldSuspend = false } = props;
    const activeDrawingId = useBuerli(state => state.drawing.active);
    const actualDrawingId = drawingId !== null && drawingId !== void 0 ? drawingId : activeDrawingId;
    if (Array.isArray(shouldSuspend) && didWarn === false) {
        console.warn('Array notation for <BuerliGeometry suspend={[".Load", ".SetDatabaseSettings", ...]}> is deprecated. Use <BuerliGeometry suspend=".Load,.SetDatabaseSettings" /> instead.');
        didWarn = true;
    }
    // Get pending states
    const pending = useDrawing(actualDrawingId, state => state.cad.pending);
    const filterItems = typeof shouldSuspend === 'string' ? shouldSuspend.split(',') : Array.isArray(shouldSuspend) ? shouldSuspend : [];
    const filter = typeof shouldSuspend === 'function'
        ? shouldSuspend
        : (data) => typeof shouldSuspend === 'boolean' ? shouldSuspend : filterItems.find(entry => data.name.includes(entry));
    // Collect all pending promises if shouldSuspend is true
    const hashes = (_a = pending === null || pending === void 0 ? void 0 : pending.filter(data => data.promise && filter(data))) !== null && _a !== void 0 ? _a : [];
    const promises = hashes.map(data => data.promise);
    // Suspend all pending promises. This prevents the rendering of the component until all promises are resolved.
    // It will also allow external components to interface with the results. A useEffect in the same <Suspense>
    // boundary will guarantee the presence of all loaded products and geometry.
    if (shouldSuspend)
        suspend(() => Promise.all(promises), ['buerli/react/BuerliGeometry', ...promises]);
    if (actualDrawingId === null)
        return null;
    return selection ? (React.createElement(BuerliGeometryInteraction, { drawingId: actualDrawingId },
        React.createElement(Product, Object.assign({}, props, { drawingId: actualDrawingId, isRoot: true })))) : (React.createElement(Product, Object.assign({}, props, { drawingId: actualDrawingId, isRoot: true })));
});
