import { MathUtils } from '@buerli.io/core';
import React from 'react';
import * as THREE from 'three';
import { useDrawing } from '../hooks';
/**
 * Goes up the object tree starting from the given objectId
 * and accumulates the transformations of all predecessors resulting in the global matrix of the given objectId.
 * These transformations are reactive, i.e. every time some subobject's matrix changes, the final transformation is recalculated as well.
 * When root is reached it wraps children into a group and applies resulting matrix to it.
 *
 * @param drawingId - id of drawing
 * @param objectId - id of object whoes parents should be taken in account
 * @param matrix - needed for recursive calls
 */
export const GlobalTransform = ({ drawingId, objectId, matrix = new THREE.Matrix4(), children }) => {
    // For some reason, this component will be rendered even if the corresponding object does
    // not exist in the tree anymore. So its existance needs to be checked!
    const parentId = useDrawing(drawingId, drawing => { var _a; return (_a = drawing.structure.tree[objectId]) === null || _a === void 0 ? void 0 : _a.parent; });
    const csys = useDrawing(drawingId, drawing => { var _a; return (_a = drawing.structure.tree[objectId]) === null || _a === void 0 ? void 0 : _a.coordinateSystem; });
    const matrix_ = React.useMemo(() => {
        if (!csys) {
            return matrix.clone();
        }
        const local = MathUtils.convertToMatrix4(csys);
        return local.multiply(matrix);
    }, [csys, matrix]);
    return parentId ? (React.createElement(GlobalTransform, { drawingId: drawingId, objectId: parentId, matrix: matrix_ }, children)) : (React.createElement("group", { matrix: matrix_, matrixAutoUpdate: false }, children));
};
