import { MathUtils } from '@buerli.io/core';
import React, { useMemo } from 'react';
import { animated as a, useTransition } from 'react-spring';
import { usePluginExists } from '../hooks/usePluginExists';
import styled from 'styled-components';
import * as THREE from 'three';
import { useDrawing } from '../hooks/useDrawing';
import { usePlugin } from '../hooks/usePlugin';
function Root({ drawingId, pluginId }) {
    const PluginRoot = usePlugin(drawingId, pluginId, plugin => plugin.Root);
    const result = PluginRoot ? (React.createElement(Container, null,
        React.createElement(PluginRoot, { drawingId: drawingId, pluginId: pluginId }))) : null;
    const transitions = useTransition(result, null, {
        from: { opacity: 0, transform: 'translate3d(0,20px,0)' },
        enter: { opacity: 1, transform: 'translate3d(0,0px,0)' },
        leave: { opacity: 0, transform: 'translate3d(0,20px,0)' },
    });
    return (React.createElement(React.Fragment, null, transitions.map(({ item, props, key }) => (React.createElement(a.div, { key: key, style: props, children: item })))));
}
export function View({ drawingId, pluginId, additionalProps = {}, }) {
    const objectId = usePlugin(drawingId, pluginId, state => state.objectId) || -1;
    const isGlobal = usePlugin(drawingId, pluginId, state => state.global);
    // Selector of useDrawing below might be called after removing object with objectId from tree
    // Thus we need to check whether the object exists in tree or not before getting coordinateSystem
    const csys = useDrawing(drawingId, drawing => { var _a; return objectId >= 0 && ((_a = drawing.structure.tree[objectId]) === null || _a === void 0 ? void 0 : _a.coordinateSystem); });
    const transformMatrix = useMemo(() => (csys ? MathUtils.convertToMatrix4(csys) : new THREE.Matrix4()), [csys]);
    // CAD-3022.
    // When feature object has been just deleted corresponding plugin still exists for a while
    // Thus this component will be rendered for unexisting object. In this case it shoud return null
    const objectExists = useDrawing(drawingId, d => d.structure.tree[objectId] !== undefined);
    const PluginView = usePlugin(drawingId, pluginId, plugin => plugin.View);
    if (!PluginView)
        return null;
    if (!objectExists && !isGlobal)
        return null;
    return (React.createElement("group", { matrix: transformMatrix, matrixAutoUpdate: false },
        React.createElement(PluginView, Object.assign({ drawingId: drawingId, pluginId: pluginId }, additionalProps))));
}
export default function Plugin({ view, drawingId, pluginId, }) {
    // Feature plugins are lazy loaded, so it's possible that rendered plugin hasn't been created yet
    const isCreated = usePluginExists(drawingId, pluginId);
    const Type = view ? View : Root;
    return isCreated ? React.createElement(Type, { key: pluginId, pluginId: pluginId, drawingId: drawingId }) : null;
}
const Container = styled.div `
  width: 100%;
  color: rgba(0, 0, 0, 0.75);
  display: flex;
  flex-direction: column;
`;
