import { GraphicType, NOID } from '@buerli.io/core';
import * as deepmerge from 'deepmerge';
import React from 'react';
import { useBuerli, useDrawing } from '../hooks';
import { Geometry, Group } from './geometry';
import { GlobalTransform } from './GlobalTransform';
// TODO: Make GlobalTransform optional, by default true
const Solid = ({ drawingId, solidId, productId, }) => {
    var _a, _b;
    const globalConfig = useBuerli(buerli => buerli.options.config.geometry);
    const geometry = useDrawing(drawingId, drawing => drawing.geometry.cache[solidId]);
    const productConfig = useDrawing(drawingId, drawing => drawing.geometry.config.product[productId]);
    const solidConfig = useDrawing(drawingId, drawing => drawing.geometry.config.product[solidId]);
    const config = React.useMemo(() => {
        return deepmerge.all([globalConfig, productConfig || {}, solidConfig || {}]);
    }, [globalConfig, productConfig, solidConfig]);
    const ownerId = (_b = (_a = geometry === null || geometry === void 0 ? void 0 : geometry.container) === null || _a === void 0 ? void 0 : _a.ownerId) !== null && _b !== void 0 ? _b : NOID;
    const ownerName = useDrawing(drawingId, drawing => { var _a; return (_a = drawing.structure.tree[ownerId]) === null || _a === void 0 ? void 0 : _a.name; });
    return geometry && geometry.type === GraphicType.BREP ? (React.createElement("group", { name: ownerName, userData: { id: ownerId } },
        React.createElement("group", { name: `${solidId}`, userData: { id: solidId } },
            React.createElement(Geometry, { key: geometry.dirtyStamp || -1, drawingId: drawingId, containerId: solidId, productId: productId, geometry: geometry, config: config })))) : null;
};
const Part = ({ drawingId, partId, productId, }) => {
    const part = useDrawing(drawingId, state => state.structure.tree[partId]);
    return part ? (React.createElement("group", { name: part.name, userData: { id: part.id } }, Array.isArray(part.solids)
        ? part.solids.map(id => React.createElement(Solid, { key: id, productId: productId, solidId: id, drawingId: drawingId }))
        : null)) : null;
};
export const Product = React.memo(({ drawingId, isRoot = false, productId, children }) => {
    const rootId = useDrawing(drawingId, state => state.structure.root);
    const objId = (isRoot && productId === undefined ? rootId : productId) || 0;
    const obj = useDrawing(drawingId, state => state.structure.tree[objId]);
    if (obj) {
        return (React.createElement(React.Fragment, null, isRoot ? (React.createElement(React.Fragment, null,
            React.createElement(GlobalTransform, { drawingId: drawingId, objectId: objId },
                React.createElement(Content, { obj: obj, drawingId: drawingId, children: children })))) : (React.createElement(Group, { coordinateSystem: obj.coordinateSystem },
            React.createElement(Content, { obj: obj, drawingId: drawingId, children: children })))));
    }
    return null;
});
function Content({ obj, drawingId, children, }) {
    let content = null;
    if (Array.isArray(obj.solids)) {
        content = React.createElement(Part, { partId: obj.id, productId: obj.id, drawingId: drawingId });
    }
    else if (typeof obj.link !== 'undefined') {
        content = React.createElement(Part, { partId: obj.link, productId: obj.id, drawingId: drawingId });
    }
    else if (obj.children) {
        content = obj.children.map(id => React.createElement(Product, { key: id, productId: id, drawingId: drawingId, children: children }));
    }
    else {
        content = null;
    }
    return children || content ? (React.createElement("group", { name: obj.name, userData: { id: obj.id } }, children ? children({ node: content, object: obj }) : content)) : null;
}
