import React from 'react';
import * as THREE from 'three';
import { mergeBufferGeometries } from 'three-stdlib';
import { PointCloud } from './PointCloud';
export const FastGeometry = props => {
    return (React.createElement("group", null,
        React.createElement(FastSolid, Object.assign({}, props)),
        React.createElement(PointCloud, Object.assign({}, props))));
};
const FastSolid = props => {
    const { productId, containerId, geometry: container, config } = props;
    const groupRef = React.useRef(null);
    const isActive = (config === null || config === void 0 ? void 0 : config.disabled) === undefined ? true : !config.disabled;
    const meshes = React.useMemo(() => {
        var _a, _b, _c, _d;
        if ((_a = config === null || config === void 0 ? void 0 : config.meshes) === null || _a === void 0 ? void 0 : _a.hidden) {
            return [];
        }
        const coloredMeshes = {};
        for (const mesh of container.meshes) {
            const hexColor = (mesh.color && mesh.color.getHexString()) || '__nocolor__';
            const meshInfo = coloredMeshes[hexColor] || {
                geoms: [],
                mm: {},
                mg: null,
                lastIndex: 0,
                color: mesh.color || container.color || ((_b = config === null || config === void 0 ? void 0 : config.meshes) === null || _b === void 0 ? void 0 : _b.color),
                opacity: isActive ? mesh.opacity || container.opacity || ((_c = config === null || config === void 0 ? void 0 : config.meshes) === null || _c === void 0 ? void 0 : _c.opacity) : 0.1,
                wireframe: ((_d = config === null || config === void 0 ? void 0 : config.meshes) === null || _d === void 0 ? void 0 : _d.wireframe) !== undefined ? config.meshes.wireframe : false,
            };
            coloredMeshes[hexColor] = meshInfo;
            meshInfo.geoms.push(mesh);
        }
        return Object.keys(coloredMeshes).map(key => {
            var _a;
            let faceIndex = 0;
            const res = coloredMeshes[key];
            for (const element of res.geoms) {
                const faceCount = (_a = element.geometry.getIndex()) === null || _a === void 0 ? void 0 : _a.count;
                if (faceCount) {
                    for (let i = 0; i < faceCount / 3; i++) {
                        res.mm[faceIndex] = element;
                        faceIndex++;
                    }
                }
            }
            res.mg = mergeBufferGeometries(res.geoms.map(g => g.geometry));
            return res;
        });
    }, [config, container.meshes, container.color, container.opacity, isActive]);
    const lines = React.useMemo(() => {
        var _a, _b, _c;
        if ((_a = config === null || config === void 0 ? void 0 : config.edges) === null || _a === void 0 ? void 0 : _a.hidden) {
            return [];
        }
        const coloredLines = {};
        // const lineInfo: LineInfo = { lm: {}, lg: null }
        const allLines = [...container.lines, ...container.edges, ...container.arcs];
        for (const line of allLines) {
            const hexColor = (line.color && line.color.getHexString()) || '__nocolor__';
            const lineInfo = coloredLines[hexColor] || {
                geoms: [],
                lm: {},
                lg: null,
                color: line.color || ((_b = config === null || config === void 0 ? void 0 : config.edges) === null || _b === void 0 ? void 0 : _b.color),
                opacity: line.opacity || ((_c = config === null || config === void 0 ? void 0 : config.edges) === null || _c === void 0 ? void 0 : _c.opacity) || 1.0,
            };
            coloredLines[hexColor] = lineInfo;
            lineInfo.geoms.push(line);
        }
        return Object.keys(coloredLines).map(key => {
            let lineIndex = 0;
            const res = coloredLines[key];
            for (const element of res.geoms) {
                const lineCount = element.geometry.attributes.position.count;
                if (lineCount) {
                    for (let i = 0; i < lineCount; i++) {
                        res.lm[lineIndex] = element;
                        lineIndex++;
                    }
                }
            }
            res.lg = mergeBufferGeometries(res.geoms.map(g => g.geometry));
            return res;
        });
    }, [config, container.arcs, container.edges, container.lines]);
    ////////////////////////
    // Return the DOM
    ////////////////////////
    return (React.createElement("group", { ref: groupRef, name: `${container.graphicId}|${productId}` },
        meshes.length > 0 &&
            meshes.map((mesh, index) => {
                var _a, _b;
                return mesh.mg ? (React.createElement("mesh", { key: index, geometry: mesh.mg, renderOrder: isActive ? 50 : 0, userData: { meshMap: mesh.mm, isBuerliGeometry: isActive, productId, containerId } },
                    React.createElement("meshLambertMaterial", { opacity: mesh.opacity, color: mesh.color, polygonOffset: true, polygonOffsetFactor: 1, polygonOffsetUnits: 1, side: isActive ? THREE.DoubleSide : THREE.FrontSide, transparent: mesh.opacity < 1, depthWrite: ((_a = config === null || config === void 0 ? void 0 : config.meshes) === null || _a === void 0 ? void 0 : _a.depthWrite) === undefined ? true : (_b = config === null || config === void 0 ? void 0 : config.meshes) === null || _b === void 0 ? void 0 : _b.depthWrite, wireframe: mesh.wireframe }))) : null;
            }),
        lines.length > 0 &&
            lines.map((line, index) => {
                return line.lg ? (React.createElement("lineSegments", { key: index, geometry: line.lg, renderOrder: isActive ? 150 : 100, userData: { lineMap: line.lm, isBuerliGeometry: isActive, productId, containerId } },
                    React.createElement("lineBasicMaterial", { transparent: line.opacity < 1, opacity: line.opacity, color: line.color }))) : null;
            })));
};
