import React from 'react';
import { useThree } from '@react-three/fiber';
import * as THREE from 'three';
import disc from './pointimage.png';
const defaultColor = 'black';
export const PointCloud = props => {
    const { geometry, productId, containerId, config } = props;
    const pointConfig = config === null || config === void 0 ? void 0 : config.points;
    const { invalidate } = useThree();
    const posAttr = React.useRef(null);
    const colorAttr = React.useRef(null);
    const isActive = (config === null || config === void 0 ? void 0 : config.disabled) === undefined ? true : !config.disabled;
    const [positions, colors, lookup] = React.useMemo(() => {
        const color = new THREE.Color((pointConfig === null || pointConfig === void 0 ? void 0 : pointConfig.color) || defaultColor);
        const positions1 = [];
        const colors1 = [];
        const lookupMap = {};
        if (geometry) {
            for (let i = 0; i < geometry.points.length; i++) {
                const pt = geometry.points[i];
                positions1.push(pt.position.x);
                positions1.push(pt.position.y);
                positions1.push(pt.position.z);
                colors1.push(color.r);
                colors1.push(color.g);
                colors1.push(color.b);
                colors1.push((pointConfig === null || pointConfig === void 0 ? void 0 : pointConfig.hidden) ? 0.0 : (pointConfig === null || pointConfig === void 0 ? void 0 : pointConfig.opacity) || 1.0); // alpha
                lookupMap[`${productId}|${pt.graphicId}|${pt.type}`] = i;
            }
        }
        return [new Float32Array(positions1), new Float32Array(colors1), lookupMap];
    }, [geometry, pointConfig, productId]);
    const [uniforms] = React.useState(() => ({
        size: { value: 10 },
        tex: { value: new THREE.TextureLoader().load(disc) },
    }));
    React.useEffect(() => {
        if (posAttr.current) {
            posAttr.current.needsUpdate = true;
        }
        if (colorAttr.current) {
            colorAttr.current.needsUpdate = true;
        }
        invalidate();
    }, [positions, colors, lookup, posAttr, colorAttr, invalidate]);
    return (React.createElement("points", { renderOrder: 400, frustumCulled: false, userData: { pointMap: geometry.points, productId, containerId, isBuerliGeometry: isActive } },
        React.createElement("bufferGeometry", null,
            React.createElement("bufferAttribute", { ref: posAttr, attach: 'attributes-position', count: positions.length / 3, array: positions, itemSize: 3 }),
            React.createElement("bufferAttribute", { ref: colorAttr, attach: 'attributes-rgba', count: colors.length / 4, array: colors, itemSize: 4 })),
        React.createElement("shaderMaterial", { attach: "material", uniforms: uniforms, vertexShader: vert, fragmentShader: frag, alphaTest: 0.5, depthWrite: false, depthTest: true, vertexColors: true, transparent: true })));
};
const vert = `uniform float size;
attribute vec4 rgba;
varying vec4 vColor;
void main() {
  gl_Position = projectionMatrix * modelViewMatrix * vec4(position, 1.0);
  gl_PointSize = size;
  vColor = rgba;
}`;
const frag = `uniform sampler2D tex;
varying vec4 vColor;
void main() {
  gl_FragColor = vColor;
	gl_FragColor = gl_FragColor * texture2D(tex, gl_PointCoord);
}`;
