import { useFrame, useThree } from '@react-three/fiber';
import React from 'react';
import { CameraHelper } from '../../../helpers/CameraHelper';
const isVector3 = (entity) => entity && entity.isVector3;
export const Point = props => {
    const { elem, color = 'black', renderOrder = 1, opacity = 1, pointSize = 6.0, onClick } = props;
    const { camera, size } = useThree();
    const pointMeshRef = React.useRef(null);
    const calcScale = React.useCallback((canvasRect) => {
        const point = elem;
        const sf = CameraHelper.calculateScaleFactor(point.position, pointSize, camera, canvasRect);
        return [sf, sf, sf];
    }, [camera, elem, pointSize]);
    const updateScale = React.useCallback((canvasRect) => {
        const sf = calcScale(canvasRect);
        pointMeshRef.current && pointMeshRef.current.scale.set(sf[0], sf[1], sf[2]);
    }, [calcScale]);
    useFrame(renderProps => updateScale(renderProps.size));
    React.useEffect(() => updateScale(size), [size, updateScale]);
    const pointGeom = elem;
    return isVector3(pointGeom.position) ? (React.createElement("mesh", { onClick: e => onClick === null || onClick === void 0 ? void 0 : onClick([elem], e), ref: pointMeshRef, scale: calcScale(size), position: pointGeom.position, userData: elem, renderOrder: renderOrder },
        React.createElement("sphereGeometry", { args: [1.2, 32, 16] }),
        React.createElement("meshBasicMaterial", { color: color, transparent: true, toneMapped: false }))) : null;
};
