import { api as buerliApi, getDrawing } from '@buerli.io/core';
import React from 'react';
import shallow from 'zustand/shallow';
import { useBuerli, useDrawing } from '../../hooks';
const PluginInstance = ({ drawingId, objId }) => {
    const activeFeature = useDrawing(drawingId, d => d.plugin.active.feature);
    const isVisible = useDrawing(drawingId, d => d.plugin.visible.includes(objId));
    const unsub = React.useRef();
    // Create plugin if it's activated and hasn't been created yet
    React.useEffect(() => {
        const drawing = getDrawing(drawingId);
        if (drawing) {
            // Find object in the structure-tree
            const obj = drawing.structure.tree[objId];
            const isCreated = drawing.plugin.refs[objId] !== undefined;
            if (obj && (activeFeature === objId || isVisible) && !isCreated) {
                // Look for a plugin that fits the description
                const plugins = buerliApi.getState().options.plugins;
                const impl = plugins && plugins[obj.class];
                if (impl) {
                    // If there's one we create a plugin and return the unsubscribe function, so that it will remove itself on unmount
                    const def = typeof impl === 'function' ? impl() : impl;
                    unsub.current = drawing.api.plugin.createPlugin(def, obj.id, drawingId, obj.id);
                }
            }
        }
    }, [drawingId, objId, activeFeature, isVisible]);
    // Remove plugin on PluginInstace component unoumt (regularly it happens when corresponding object is removed from tree)
    React.useEffect(() => {
        return () => {
            const unsub_ = unsub.current;
            const drawing = getDrawing(drawingId);
            if (unsub_ && drawing) {
                unsub_(objId === drawing.plugin.active.feature);
            }
        };
    }, [drawingId, objId]);
    // This component has no view
    return null;
};
const PluginDrawing = ({ drawingId }) => {
    const ids = useDrawing(drawingId, state => Object.getOwnPropertyNames(state.structure.tree), shallow);
    return (React.createElement(React.Fragment, null, ids === null || ids === void 0 ? void 0 : ids.map(id => (React.createElement(PluginInstance, { key: id, drawingId: drawingId, objId: Number(id) })))));
};
const PluginManagerImpl = () => {
    const drawingIds = useBuerli(state => state.drawing.ids);
    return (React.createElement(React.Fragment, null, drawingIds.map(drawingId => (React.createElement(PluginDrawing, { key: drawingId, drawingId: drawingId })))));
};
export const PluginManager = React.memo(PluginManagerImpl);
