import { getDrawing } from '@buerli.io/core';
import React from 'react';
import { useDrawing, usePlugin } from '../../hooks';
import Plugin from './../Plugin';
const useAncestors = (drawingId, objectId) => {
    return React.useMemo(() => {
        const drawing = getDrawing(drawingId);
        if (!objectId || !drawing) {
            return [];
        }
        const tree = drawing.structure.tree;
        let curId = objectId;
        const ancestors = [];
        while (Boolean(curId) && tree[curId]) {
            ancestors.push(tree[curId].parent);
            curId = tree[curId].parent;
        }
        return ancestors;
    }, [drawingId, objectId]);
};
export const PluginWrapperView = props => {
    const { drawingId, pluginId, showOnlyActive = true } = props;
    const objectId = usePlugin(drawingId, pluginId, plugin => plugin.objectId);
    const ancestors = useAncestors(drawingId, objectId);
    const currentProduct = useDrawing(drawingId, d => d.structure.currentProduct) || -1;
    const isActive = ancestors.indexOf(currentProduct) !== -1;
    const visible = useDrawing(drawingId, d => d.plugin.visible) || [];
    const globPlugins = useDrawing(drawingId, d => d.plugin.active.global) || [];
    const featurePlugin = useDrawing(drawingId, d => d.plugin.active.feature);
    const useIt = showOnlyActive ? isActive : true;
    const showIt = (useIt && visible.indexOf(pluginId) >= 0) || globPlugins.indexOf(pluginId) >= 0 || pluginId === featurePlugin;
    return showIt ? React.createElement(Plugin, { drawingId: drawingId, pluginId: pluginId, view: true }) : null;
};
