import { LoadingOutlined } from '@ant-design/icons';
import notification from 'antd/lib/notification';
import React from 'react';
import { useBuerli } from '../../hooks';
const defaultStyle = {
    margin: 5,
    padding: 3,
    borderRadius: 5,
};
const defaultConfig = {
    placement: 'bottomRight',
};
const showNotification = (msg, id, style) => {
    const args = {
        key: id,
        message: msg.title ? msg.title : msg.type,
        description: msg.text.length > 120 ? msg.text.substring(0, 120) + ' ...' : msg.text,
        style,
    };
    switch (msg.type) {
        case 'error':
            args.duration = 0;
            notification.error(args);
            break;
        case 'info':
            notification.info(args);
            break;
        case 'busy':
            args.icon = React.createElement(LoadingOutlined, { style: { fontSize: 24 }, spin: true });
            args.duration = 0;
            notification.info(args);
            break;
        case 'success':
            notification.success(args);
            break;
        case 'warning':
            notification.warning(args);
            break;
        default:
            notification.open(args);
            break;
    }
};
export const Messenger = ({ config = defaultConfig, style = defaultStyle, }) => {
    const ts = useBuerli(state => state.message.timestamp);
    const msg = useBuerli(state => state.message.data);
    React.useEffect(() => {
        notification.config(config);
    }, [config]);
    React.useMemo(() => {
        msg && (msg.text || msg.title) && showNotification(msg, msg.id, style);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [ts, msg, style]);
    return null;
};
export default Messenger;
