import { useBuerli } from './useBuerli';
/**
 * Hook to select parts of the drawing state.
 *
 * @param id The drawing id.
 * @param sel The selector of the drawing state.
 * @param equalityFn The equality function.
 * @returns The part of the drawing state selected by the selector function (`sel`). Result can be undefined!
 */
export function useDrawing(id, sel, equalityFn) {
    return useBuerli(state => {
        try {
            if (state.drawing.refs[id])
                return sel(state.drawing.refs[id]);
        }
        catch (e) {
            console.warn(e);
        }
        return undefined;
    }, equalityFn);
}
