import { getDrawing } from '@buerli.io/core';
import React, { useEffect, useState } from 'react';
import { View } from '../components/Plugin';
/**
 * UseEmbeddedPlugin might be called twice for the same plugin,
 * but embedded plugin should be created and remove once.
 * Creating plugin once - can be implemented by checking of plugin existing.
 * For removing plugin once we hanve to count usages of this plugin,
 * and when counter equals to 0 remove plugin from store.
 *
 * Increasing and decreasing usageCounter happens in last UseEffect.
 **/
const usageCounters = {};
const unsubs = {};
export function useEmbeddedPlugin(definition, drawingId, parentPluginId, objectId, onCreated) {
    const [attributes] = useState(() => ({
        drawingId,
        objectId,
        pluginId: parentPluginId + '/embed/' + definition.description.name,
    }));
    // Fetch plugin from store
    const plugin = getDrawing(drawingId).plugin.refs[attributes.pluginId];
    // If it exists already, we are ready to roll ...
    const [ready, setReady] = useState(!!plugin);
    // If not we have to create it
    useEffect(() => {
        const plugin_ = getDrawing(drawingId).plugin.refs[attributes.pluginId];
        if (!plugin_) {
            // Create plugin
            const pluginApi = getDrawing(drawingId).api.plugin;
            const unsub = pluginApi.createPlugin(definition, attributes.pluginId, drawingId, objectId);
            usageCounters[attributes.pluginId] = 0;
            // Flag ready state
            unsubs[attributes.pluginId] = unsub;
        }
        // Here plugin exists and now we have to render it.
        setReady(true);
    }, [definition, drawingId, objectId, plugin, attributes.pluginId]);
    useEffect(() => {
        // Increase referece count on mount.
        usageCounters[attributes.pluginId]++;
        return () => {
            // Decrease referece count on mount and remove plugin if refCount === 0.
            if (--usageCounters[attributes.pluginId] === 0) {
                unsubs[attributes.pluginId](false);
                delete unsubs[attributes.pluginId];
            }
        };
    }, [drawingId, attributes.pluginId]);
    const onCreatedRef = React.useRef(onCreated);
    useEffect(() => {
        onCreatedRef.current && onCreatedRef.current(attributes.pluginId);
    }, [attributes.pluginId]);
    // Return plugins root and view
    return [
        (props) => (ready && definition.Root ? React.createElement(definition.Root, Object.assign({}, props, attributes)) : null),
        (props) => (ready && definition.View ? React.createElement(View, Object.assign({ additionalProps: props }, attributes)) : null),
    ];
}
