import { DrawingID, ObjectID } from '@buerli.io/core';
import React from 'react';
import * as THREE from 'three';
/**
 * Goes up the object tree starting from the given objectId
 * and accumulates the transformations of all predecessors resulting in the global matrix of the given objectId.
 * These transformations are reactive, i.e. every time some subobject's matrix changes, the final transformation is recalculated as well.
 * When root is reached it wraps children into a group and applies resulting matrix to it.
 *
 * @param drawingId - id of drawing
 * @param objectId - id of object whoes parents should be taken in account
 * @param matrix - needed for recursive calls
 */
export declare const GlobalTransform: React.FC<{
    drawingId: DrawingID;
    objectId: ObjectID;
    matrix?: THREE.Matrix4;
    children?: React.ReactNode;
}>;
