import * as THREE from 'three';
import { Size } from '@react-three/fiber';
export declare const CameraHelper: {
    /**
     * Projects 2-dimensional coordinate from a 3-dimensional point within the view's scene.
     * @param {THREE.Vector3} point3 - Input point
     * @example
     * // Grab x and y off THREE's projected Vector2
     * let {x, y} = view.getPoint2D(new THREE.Vector3(10, 20, 100));
     * @returns {THREE.Vector2} The projected point
     */
    getPoint2(point3: THREE.Vector3, camera: THREE.Camera, size: Size): THREE.Vector3;
    /**
     * Projects a 2D point into 3D space. Z-Value for the 2D-Point can be passed within bounds of 0 to 1.
     * Note: maximal depth is used by default (z = 1), so the point is on the "far" frustum (with huge coordinates)
     */
    getPoint3(point2: THREE.Vector3, camera: THREE.Camera, size: Size, zValue?: number): THREE.Vector3;
    /**
     * Calculate the scale factor that will make the object at the point3 position be radiusPx pixels wide.
     * Example use: object.scale.set(calculateScaleFactor(object.getWorldPosition(), radiusPx));
     * @param point3   Position of the object
     * @param radiusPx Desired radius of the object in pixels
     * @param camera   The camera
     * @param size     The size of the canvas
     * @return         Scale factor for the object
     */
    calculateScaleFactor(point3: THREE.Vector3, radiusPx: number, camera: THREE.Camera, size: Size): number;
    calculateNearFar: (camera: THREE.OrthographicCamera, box: THREE.Box3) => {
        near: number;
        far: number;
    };
};
